# -*- coding: utf-8 -*-

'''
Animec
~~~~~~~~~~~~~~~~~~~
A module to get data about anime characters, news, info, lyrics and more.
'''

from .sagasu import *
from .aninews import *
from .anicore import *
from .errors import *

__author__ = "DriftAsimov"
__title__ = "animec"
__copyright__ = "Copyright Drift Asimov 2021"
__version__ = "0.2.0"
__license__ = "MIT"
